@@ -1,5 +1,5 @@
 /* mp4h -- A macro processor for HTML documents
-   Copyright 2000, Denis Barbier
+   Copyright 2000-2003, Denis Barbier
    All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
@@ -32,7 +32,7 @@
 
 #include <sys/stat.h>
 
-#if __STDC__
+#ifdef HAVE_STDARG_H
 #include <stdarg.h>
 #else
 #include <varargs.h>
@@ -123,6 +123,10 @@ debug_decode (const char *opts)
               level |= DEBUG_TRACE_CALLID;
               break;
 
+            case 'm':
+              level |= DEBUG_TRACE_MODULES;
+              break;
+
             case 'V':
               level |= DEBUG_TRACE_VERBOSE;
               break;
@@ -238,7 +242,7 @@ debug_message_prefix (void)
 | function trace_format ().  Understands only %S, %s and %d.           |
 `---------------------------------------------------------------------*/
 
-#if __STDC__
+#ifdef HAVE_STDARG_H
 static void
 trace_format (const char *fmt, ...)
 #else
@@ -246,7 +250,7 @@ static void
 trace_format (...)
 #endif
 {
-#if ! __STDC__
+#ifndef HAVE_STDARG_H
   const char *fmt;
 #endif
   va_list args;
@@ -254,11 +258,11 @@ trace_format (...)
 
   int d;
   char nbuf[32];
-  const char *s;
+  char *s;
   int slen;
   int maxlen;
 
-#if __STDC__
+#ifdef HAVE_STDARG_H
   va_start (args, fmt);
 #else
   va_start (args);
@@ -281,7 +285,7 @@ trace_format (...)
           /* fall through */
 
         case 's':
-          s = va_arg (args, const char *);
+          s = va_arg (args, char *);
           break;
 
         case 'd':
@@ -295,6 +299,8 @@ trace_format (...)
           break;
         }
 
+      /*   Remove special chars from output */
+      remove_special_chars (s, TRUE);
       slen = strlen (s);
       if (maxlen == 0 || maxlen > slen)
         obstack_grow (&trace, s, slen);
@@ -383,24 +389,24 @@ trace_pre (const char *name, int id, int argc, token_data **argv)
               text = xstrdup (TOKEN_DATA_TEXT (argv[i]));
               remove_special_chars (text, FALSE);
               trace_format ("%S", text);
-              xfree (text);
+              xfree ((voidstar) text);
               break;
 
             case TOKEN_FUNC:
               bp = find_builtin_by_addr (TOKEN_DATA_FUNC (argv[i]));
               if (bp == NULL)
                 {
-                  MP4HERROR ((warning_status, 0, _("\
-INTERNAL ERROR: Builtin not found in builtin table! (trace_pre ())")));
-                  abort ();
+                  MP4HERROR ((warning_status, 0, "\
+INTERNAL ERROR: Builtin not found in builtin table! (trace_pre ())"));
+                  exit (1);
                 }
               trace_format ("%s(prim)", bp->name);
               break;
 
             default:
-              MP4HERROR ((warning_status, 0, _("\
-INTERNAL ERROR: Bad token data type (trace_pre ())")));
-              abort ();
+              MP4HERROR ((warning_status, 0, "\
+INTERNAL ERROR: Bad token data type (trace_pre ())"));
+              exit (1);
             }
 
         }
@@ -436,7 +442,7 @@ trace_post (const char *name, int id, int argc, token_data **argv,
       text = xstrdup (expanded);
       remove_special_chars (text, FALSE);
       trace_format (" -> %S", text);
-      xfree (text);
+      xfree ((voidstar) text);
     }
   trace_flush ();
 }
