@@ -1,5 +1,5 @@
 /* mp4h -- A macro processor for HTML documents
-   Copyright 2000-2001, Denis Barbier
+   Copyright 2000-2003, Denis Barbier
    All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
@@ -50,6 +50,9 @@
 /* Pointer to symbol table.  */
 symbol **sym_tab;
 
+/* Pointer to entity table.  */
+symbol **entity_tab;
+
 /* Pointer to variable table.  */
 symbol **var_tab;
 
@@ -59,6 +62,15 @@ symbol **file_tab;
 /* Any of previous pointers.  */
 symbol **symtab;
 
+/* True if tags are case insensitive  */
+boolean caseless_tag;
+
+/* True if variables are case insensitive  */
+boolean caseless_var;
+
+/* True if entities are case insensitive  */
+boolean caseless_ent;
+
 static void
 hash_table_init (symbol **s)
 {
@@ -72,13 +84,23 @@ symtab_init (void)
 {
   var_tab = (symbol **) xmalloc (hash_table_size * sizeof (symbol *));
   file_tab = (symbol **) xmalloc (hash_table_size * sizeof (symbol *));
+  entity_tab = (symbol **) xmalloc (hash_table_size * sizeof (symbol *));
   sym_tab = (symbol **) xmalloc (hash_table_size * sizeof (symbol *));
   hash_table_init (var_tab);
   hash_table_init (file_tab);
+  hash_table_init (entity_tab);
   hash_table_init (sym_tab);
   symtab = sym_tab;
 }
 
+void
+caseless_init (int caseless)
+{
+  caseless_tag = ((caseless & 1) == 1);
+  caseless_var = ((caseless & 2) == 2);
+  caseless_ent = ((caseless & 4) == 4);
+}
+
 /*--------------------------------------------.
 | Free all storage associated with a symbol.  |
 `--------------------------------------------*/
@@ -118,6 +140,7 @@ symtab_deallocate (void)
 {
   hash_table_free (var_tab);
   hash_table_free (file_tab);
+  hash_table_free (entity_tab);
   hash_table_free (sym_tab);
 }
 
@@ -157,7 +180,7 @@ hash (const char *s)
 `------------------------------------------------------------------------*/
 
 static symbol *
-generic_lookup (const char *name, symbol_lookup mode)
+generic_lookup (const char *name, symbol_lookup mode, boolean caseless)
 {
   int h, cmp = 1;
   symbol *sym, *prev;
@@ -166,8 +189,11 @@ generic_lookup (const char *name, symbol_lookup mode)
   char *cp;
 
   lcname = xstrdup (name);
-  for (cp=lcname; *cp != '\0'; cp++)
-    *cp = tolower (*cp);
+  if (caseless)
+    {
+      for (cp=lcname; *cp != '\0'; cp++)
+        *cp = tolower (*cp);
+    }
 
   h = hash (lcname);
   sym = symtab[h];
@@ -232,8 +258,8 @@ generic_lookup (const char *name, symbol_lookup mode)
 
     default:
       MP4HERROR ((warning_status, 0,
-        _("INTERNAL ERROR: Illegal mode to symbol_lookup ()")));
-      abort ();
+        "INTERNAL ERROR: Illegal mode to symbol_lookup ()"));
+      exit (1);
     }
   xfree ((voidstar) lcname);
   return sym;
@@ -243,21 +269,28 @@ symbol *
 lookup_symbol (const char *name, symbol_lookup mode)
 {
   symtab = sym_tab;
-  return generic_lookup (name, mode);
+  return generic_lookup (name, mode, caseless_tag);
+}
+
+symbol *
+lookup_entity (const char *name, symbol_lookup mode)
+{
+  symtab = entity_tab;
+  return generic_lookup (name, mode, caseless_ent);
 }
 
 symbol *
 lookup_variable (const char *name, symbol_lookup mode)
 {
   symtab = var_tab;
-  return generic_lookup (name, mode);
+  return generic_lookup (name, mode, caseless_var);
 }
 
 symbol *
 lookup_file (const char *name, symbol_lookup mode)
 {
   symtab = file_tab;
-  return generic_lookup (name, mode);
+  return generic_lookup (name, mode, FALSE);
 }
 
 /*----------------------------------------------------------------------.
@@ -317,7 +350,10 @@ symtab_debug (void)
       s = lookup_symbol (text, SYMBOL_LOOKUP);
 
       if (s == NULL)
-        printf (_("Name `%s' is unknown\n"), text);
+        {
+          printf (_("Name `%s' is unknown"), text);
+          printf ("\n");
+        }
 
       if (delete)
         (void) lookup_symbol (text, SYMBOL_DELETE);
